@@ -31,13 +31,10 @@
 #include <QMainWindow>
 #include <QPointer>
 
-#ifndef Q_OS_MACOS
-#include <QSystemTrayIcon>
-#endif
-
 #include "base/bittorrent/torrent.h"
 #include "base/logger.h"
 #include "base/settingvalue.h"
+#include "guiapplicationcomponent.h"
 
 class QCloseEvent;
 class QFileSystemWatcher;
@@ -71,12 +68,24 @@ namespace Ui
     class MainWindow;
 }
 
-class MainWindow final : public QMainWindow
+#if (defined(Q_OS_UNIX) && !defined(Q_OS_MACOS)) && defined(QT_DBUS_LIB)
+#define QBT_USES_CUSTOMDBUSNOTIFICATIONS
+class DBusNotifier;
+#endif
+
+class MainWindow final : public QMainWindow, public GUIApplicationComponent
 {
     Q_OBJECT
+    Q_DISABLE_COPY_MOVE(MainWindow)
 
 public:
-    explicit MainWindow(QWidget *parent = nullptr);
+    enum State
+    {
+        Normal,
+        Minimized
+    };
+
+    explicit MainWindow(IGUIApplication *app, State initialState = Normal);
     ~MainWindow() override;
 
     QWidget *currentTabWidget() const;
@@ -90,14 +99,6 @@ public:
     void setExecutionLogMsgTypes(Log::MsgTypes value);
 
     // Notifications properties
-    bool isNotificationsEnabled() const;
-    void setNotificationsEnabled(bool value);
-    bool isTorrentAddedNotificationsEnabled() const;
-    void setTorrentAddedNotificationsEnabled(bool value);
-#if (defined(Q_OS_UNIX) && !defined(Q_OS_MACOS)) && defined(QT_DBUS_LIB)
-    int getNotificationTimeout() const;
-    void setNotificationTimeout(int value);
-#endif
 
     // Misc properties
     bool isDownloadTrackerFavicon() const;
@@ -106,18 +107,12 @@ public:
     void activate();
     void cleanup();
 
-    void showNotificationBalloon(const QString &title, const QString &msg) const;
-
-signals:
-    void systemTrayIconCreated();
-
 private slots:
-    void showFilterContextMenu(const QPoint &);
-    void balloonClicked();
-    void writeSettings();
-    void readSettings();
-    void fullDiskError(BitTorrent::Torrent *const torrent, const QString &msg) const;
-    void handleDownloadFromUrlFailure(const QString &, const QString &) const;
+    void showFilterContextMenu();
+    void desktopNotificationClicked();
+    void saveSettings() const;
+    void loadSettings();
+    void saveSplitterSettings() const;
     void tabChanged(int newTab);
     bool defineUILockPassword();
     void clearUILockPassword();
@@ -135,10 +130,7 @@ private slots:
     void reloadSessionStats();
     void reloadTorrentStats(const QVector<BitTorrent::Torrent *> &torrents);
     void loadPreferences();
-    void addTorrentFailed(const QString &error) const;
-    void torrentNew(BitTorrent::Torrent *const torrent) const;
-    void finishedTorrent(BitTorrent::Torrent *const torrent) const;
-    void askRecursiveTorrentDownloadConfirmation(BitTorrent::Torrent *const torrent);
+    void askRecursiveTorrentDownloadConfirmation(const BitTorrent::Torrent *torrent);
     void optionsSaved();
     void toggleAlternativeSpeeds();
 
@@ -158,6 +150,7 @@ private slots:
     void on_actionSpeedInTitleBar_triggered();
     void on_actionTopToolBar_triggered();
     void on_actionShowStatusbar_triggered();
+    void on_actionShowFiltersSidebar_triggered(bool checked);
     void on_actionDonateMoney_triggered();
     void on_actionExecutionLogs_triggered(bool checked);
     void on_actionNormalMessages_triggered(bool checked);
@@ -181,7 +174,7 @@ private slots:
     // Check for unpaused downloading or seeding torrents and prevent system suspend/sleep according to preferences
     void updatePowerManagementState();
 
-    void toolbarMenuRequested(const QPoint &point);
+    void toolbarMenuRequested();
     void toolbarIconsOnly();
     void toolbarTextOnly();
     void toolbarTextBeside();
@@ -190,16 +183,11 @@ private slots:
 #ifdef Q_OS_MACOS
     void on_actionCloseWindow_triggered();
 #else
-    void toggleVisibility(const QSystemTrayIcon::ActivationReason reason = QSystemTrayIcon::Trigger);
+    void toggleVisibility();
 #endif
 
 private:
-    void createTrayIconMenu();
-#ifdef Q_OS_MACOS
-    void setupDockClickHandler();
-#else
-    void createTrayIcon(int retries);
-#endif
+    QMenu *createDesktopIntegrationMenu();
 #ifdef Q_OS_WIN
     void installPython();
 #endif
@@ -212,12 +200,13 @@ private:
     bool event(QEvent *e) override;
     void displayRSSTab(bool enable);
     void displaySearchTab(bool enable);
-    void createTorrentTriggered(const QString &path = {});
+    void createTorrentTriggered(const Path &path);
     void showStatusBar(bool show);
+    void showFiltersSidebar(bool show);
 
-    Ui::MainWindow *m_ui;
+    Ui::MainWindow *m_ui = nullptr;
 
-    QFileSystemWatcher *m_executableWatcher;
+    QFileSystemWatcher *m_executableWatcher = nullptr;
     // GUI related
     bool m_posInitialized = false;
     QPointer<QTabWidget> m_tabs;
@@ -228,43 +217,34 @@ private:
     QPointer<TorrentCreatorDialog> m_createTorrentDlg;
     QPointer<DownloadFromURLDialog> m_downloadFromURLDialog;
 
-#ifndef Q_OS_MACOS
-    QPointer<QSystemTrayIcon> m_systrayIcon;
-#endif
     QPointer<QMenu> m_trayIconMenu;
 
-    TransferListWidget *m_transferListWidget;
-    TransferListFiltersWidget *m_transferListFiltersWidget;
-    PropertiesWidget *m_propertiesWidget;
-    bool m_displaySpeedInTitle;
+    TransferListWidget *m_transferListWidget = nullptr;
+    TransferListFiltersWidget *m_transferListFiltersWidget = nullptr;
+    PropertiesWidget *m_propertiesWidget = nullptr;
+    bool m_displaySpeedInTitle = false;
     bool m_forceExit = false;
-    bool m_uiLocked;
+    bool m_uiLocked = false;
     bool m_unlockDlgShowing = false;
-    LineEdit *m_searchFilter;
-    QAction *m_searchFilterAction;
+    LineEdit *m_searchFilter = nullptr;
+    QAction *m_searchFilterAction = nullptr;
     // Widgets
-    QAction *m_queueSeparator;
-    QAction *m_queueSeparatorMenu;
-    QSplitter *m_splitter;
+    QAction *m_queueSeparator = nullptr;
+    QAction *m_queueSeparatorMenu = nullptr;
+    QSplitter *m_splitter = nullptr;
     QPointer<SearchWidget> m_searchWidget;
     QPointer<RSSWidget> m_rssWidget;
     QPointer<ExecutionLogWidget> m_executionLog;
     // Power Management
-    PowerManagement *m_pwr;
-    QTimer *m_preventTimer;
+    PowerManagement *m_pwr = nullptr;
+    QTimer *m_preventTimer = nullptr;
     bool m_hasPython = false;
-    QMenu *m_toolbarMenu;
+    QMenu *m_toolbarMenu = nullptr;
 
     SettingValue<bool> m_storeExecutionLogEnabled;
     SettingValue<bool> m_storeDownloadTrackerFavicon;
-    SettingValue<bool> m_storeNotificationEnabled;
-    SettingValue<bool> m_storeNotificationTorrentAdded;
     CachedSettingValue<Log::MsgTypes> m_storeExecutionLogTypes;
 
-#if (defined(Q_OS_UNIX) && !defined(Q_OS_MACOS)) && defined(QT_DBUS_LIB)
-    SettingValue<int> m_storeNotificationTimeOut;
-#endif
-
 #if defined(Q_OS_WIN) || defined(Q_OS_MACOS)
     void checkProgramUpdate(bool invokedByUser);
     void handleUpdateCheckFinished(ProgramUpdater *updater, bool invokedByUser);
