@@ -33,7 +33,6 @@
 #include <QVector>
 
 #include "base/bittorrent/session.h"
-#include "base/bittorrent/torrent.h"
 #include "base/global.h"
 #include "uithememanager.h"
 
@@ -41,14 +40,14 @@ namespace
 {
     QString getSpecialAllTag()
     {
-        const QString ALL_TAG = QLatin1String(" ");
+        const QString ALL_TAG = u" "_qs;
         Q_ASSERT(!BitTorrent::Session::isValidTag(ALL_TAG));
         return ALL_TAG;
     }
 
     QString getSpecialUntaggedTag()
     {
-        const QString UNTAGGED_TAG = QLatin1String("  ");
+        const QString UNTAGGED_TAG = u"  "_qs;
         Q_ASSERT(!BitTorrent::Session::isValidTag(UNTAGGED_TAG));
         return UNTAGGED_TAG;
     }
@@ -99,7 +98,7 @@ TagFilterModel::TagFilterModel(QObject *parent)
     connect(session, &Session::tagRemoved, this, &TagFilterModel::tagRemoved);
     connect(session, &Session::torrentTagAdded, this, &TagFilterModel::torrentTagAdded);
     connect(session, &Session::torrentTagRemoved, this, &TagFilterModel::torrentTagRemoved);
-    connect(session, &Session::torrentLoaded, this, &TagFilterModel::torrentAdded);
+    connect(session, &Session::torrentsLoaded, this, &TagFilterModel::torrentsLoaded);
     connect(session, &Session::torrentAboutToBeRemoved, this, &TagFilterModel::torrentAboutToBeRemoved);
     populate();
 }
@@ -124,10 +123,9 @@ QVariant TagFilterModel::data(const QModelIndex &index, int role) const
     switch (role)
     {
     case Qt::DecorationRole:
-        return UIThemeManager::instance()->getIcon("inode-directory");
+        return UIThemeManager::instance()->getIcon(u"tags"_qs);
     case Qt::DisplayRole:
-        return QString::fromLatin1("%1 (%2)")
-               .arg(tagDisplayName(item.tag())).arg(item.torrentsCount());
+        return u"%1 (%2)"_qs.arg(tagDisplayName(item.tag())).arg(item.torrentsCount());
     case Qt::UserRole:
         return item.torrentsCount();
     default:
@@ -231,16 +229,19 @@ void TagFilterModel::torrentTagRemoved(BitTorrent::Torrent *const torrent, const
     emit dataChanged(i, i);
 }
 
-void TagFilterModel::torrentAdded(BitTorrent::Torrent *const torrent)
+void TagFilterModel::torrentsLoaded(const QVector<BitTorrent::Torrent *> &torrents)
 {
-    allTagsItem()->increaseTorrentsCount();
+    for (const BitTorrent::Torrent *torrent : torrents)
+    {
+        allTagsItem()->increaseTorrentsCount();
 
-    const QVector<TagModelItem *> items = findItems(torrent->tags());
-    if (items.isEmpty())
-        untaggedItem()->increaseTorrentsCount();
+        const QVector<TagModelItem *> items = findItems(torrent->tags());
+        if (items.isEmpty())
+            untaggedItem()->increaseTorrentsCount();
 
-    for (TagModelItem *item : items)
-        item->increaseTorrentsCount();
+        for (TagModelItem *item : items)
+            item->increaseTorrentsCount();
+    }
 }
 
 void TagFilterModel::torrentAboutToBeRemoved(BitTorrent::Torrent *const torrent)
@@ -324,7 +325,7 @@ QVector<TagModelItem *> TagFilterModel::findItems(const TagSet &tags)
         if (item)
             items.push_back(item);
         else
-            qWarning() << QString::fromLatin1("Requested tag '%1' missing from the model.").arg(tag);
+            qWarning() << u"Requested tag '%1' missing from the model."_qs.arg(tag);
     }
     return items;
 }
